{% extends "explorer/base.html" %}

{% block sql_explorer_content %}
    <form method="post">{% csrf_token %}
        <div class="alert alert-info">
            Are you sure you want to delete "{{ object.title }}" ?
        </div>
        <div>
            <input type="submit" value="Delete Query" class="btn btn-default" />
        </div>
    </form>
{%  endblock %}
