{% extends "explorer/base.html" %}


{% block sql_explorer_content_takeover %}

  <div class="schema-wrapper">
    <h4 class="text-center">Schema is building...</h4>
    <div>Please wait a minute, and refresh.</div>
  </div>

{% endblock %}